/*
 * adl_status.c -- get status information from an ADL7200 device
 *
 * Copyright (C) 1998   rubini@linux.it (Alessandro Rubini)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "adl7200.h"

int main(int argc, char **argv)
{
#if 0
    int fd=0, len;
    char *fname="/dev/pxc0ctl";
    struct pxc_info *ptr;
    struct pxc_info2 *ptr2;
    unsigned int datum, datum2;
    int o; /* odd offset */
    
    if (argc>2) {
        fprintf(stderr,"%s: Usage: \"%s <filename>\"\n"
                "\tfilename should embed binary pci configuration data\n"
                "\tif filename is missing, stdin is used\n",
                argv[0],argv[0]);
        exit(1);
    }
    if (argc==2) {
	fname=argv[1];
    }

    fd=open(fname,O_RDONLY);
    if (fd<0) {
	fprintf(stderr,"%s: %s: %s\n",argv[0],fname,strerror(errno));
	exit(1);
    }

    switch (len=read(fd,buffer,BT848_NR_REGISTERS)) {
      case 0:
        fprintf(stderr,"%s: %s: no data\n",argv[0],argv[1]);
        exit(1);
      case -1:
        fprintf(stderr,"%s: %s: %s\n",argv[0],argv[1],strerror(errno));
        exit(1);
      default: break;
    }
    if (len < BT848_NR_REGISTERS) {
        buffer[len]='\0';
        fprintf(stderr,"%s: %s: short read\n",argv[0],fname);
        exit(1);
    }

#if 0
    for (len=0; len<BT848_NR_REGISTERS; ) {
	printf("0x%02x",buffer[len]&0xff);
	len++;
	putchar(len%16? ' ':'\n');
    }
#endif

    /* First of all, single-register values */

    for (ptr = dumpinfo; ptr->string; ptr++) {
	/*
	 * Perform a little-endian read of the item.
	 */
	datum = buffer[ptr->offset]
	    | (buffer[ptr->offset+1]<<8)
		| (buffer[ptr->offset+2]<<16)
		    | (buffer[ptr->offset+3]<<24);
	datum = (datum & ptr->bitmask) >> ptr->shift;
	if (ptr->type=='b')
	    printf(ptr->string, datum ? 'y' : 'n');
	else
	    printf(ptr->string, datum,
		   ptr->strings ? ptr->strings[datum] : NULL);

        /* If ptr->odd is not 0, repeat for other field */
        if (ptr->odd) {
            o=ptr->odd;
            datum = buffer[ptr->offset+o]
                | (buffer[ptr->offset+o+1]<<8)
                    | (buffer[ptr->offset+o+2]<<16)
                        | (buffer[ptr->offset+o+3]<<24);
            datum = (datum & ptr->bitmask) >> ptr->shift;
            if (ptr->type=='b')
                printf(" %c", datum ? 'y' : 'n');
            else
                printf(ptr->strings ? " -- %i: %s" : " %i", datum,
		   ptr->strings ? ptr->strings[datum] : NULL);
        }
        putchar('\n');
    }

    /* Then, two-register counters */
    for (ptr2 = dumpinfo2; ptr2->string; ptr2++) {
	/*
	 * Perform a little-endian read of the items
	 */
	datum = buffer[ptr2->offset]
	    | (buffer[ptr2->offset+1]<<8)
		| (buffer[ptr2->offset+2]<<16)
		    | (buffer[ptr2->offset+3]<<24);
	datum2 = buffer[ptr2->offset2]
	    | (buffer[ptr2->offset2+1]<<8)
		| (buffer[ptr2->offset2+2]<<16)
		    | (buffer[ptr2->offset2+3]<<24);
        datum = (((datum & ptr2->bitmask) >> ptr2->shift) << 8)
            | ((datum2 & ptr2->bitmask2) >> ptr2->shift2);
        printf(ptr2->string, datum);

        /* If ptr->odd is not 0, repeat for other field */
        if (ptr2->odd) {
            o=ptr2->odd;
            datum = buffer[ptr2->offset+o]
                | (buffer[ptr2->offset+o+1]<<8)
                    | (buffer[ptr2->offset+o+2]<<16)
                        | (buffer[ptr2->offset+o+3]<<24);
            datum2 = buffer[ptr2->offset2+o]
                | (buffer[ptr2->offset2+o+1]<<8)
                    | (buffer[ptr2->offset2+o+2]<<16)
                        | (buffer[ptr2->offset2+o+3]<<24);
            datum = (((datum & ptr2->bitmask) >> ptr2->shift) << 8)
                | ((datum2 & ptr2->bitmask2) >> ptr2->shift2);
            printf(" %i", datum);
        }
        putchar('\n');
    }
#endif
    return 0;
}

